import importlib
import os

USE_TZ = True

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

SECRET_KEY = "django-seriously-insecure"

DEBUG = True

INSTALLED_APPS = [
    "django_migrations_ci.tests.testapp",
    "django_migrations_ci",
]

_DATABASES_MODULE = os.getenv("DATABASES_MODULE", "sqlite")
DATABASES_MODULE = f"django_migrations_ci.tests.testapp.settings_{_DATABASES_MODULE}"
DATABASES = importlib.import_module(DATABASES_MODULE).DATABASES



from pathlib import Path

# Build paths inside the project like this: BASE_DIR / "subdir".
BASE_DIR = Path(__file__).resolve().parent.parent

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
        "TEST": {
            "NAME": "dbtest.sqlite3",
        },
    },
}



import os

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.mysql",
        "NAME": os.getenv("MYSQL_DB", "mysql"),
        "USER": os.getenv("MYSQL_USER", "root"),
        "PASSWORD": os.getenv("MYSQL_PASSWORD", ""),
        "HOST": os.getenv("MYSQL_HOST", "127.0.0.1"),
        "PORT": os.getenv("MYSQL_PORT", "3306"),
    },
}



def dump(connection, output_file):
    connection.ensure_connection()
    sql = "".join(f"{sql}\n" for sql in connection.connection.iterdump())

    db_name = connection.settings_dict["NAME"]
    print(f"Dump {db_name} SQL to {output_file}")
    with open(output_file, "w") as f:
        f.write(sql)



import shutil
from pathlib import Path

from django.core.management import execute_from_command_line
from django.db import connections
from django.db.utils import OperationalError
import pytest

from django_migrations_ci import django


def _check_db(connection, suffix=""):
    with django.test_db(connection, suffix=suffix):
        with connection.cursor() as conn:
            conn.execute("SELECT * FROM testapp_bus")
            result = conn.fetchall()
    assert list(result) == [(1, "BUS3R")]


def test_migrateci():
    execute_from_command_line(["manage.py", "migrateci"])
    _check_db(connections["default"])


def test_migrateci_parallel():
    execute_from_command_line(["manage.py", "migrateci", "--parallel", "1"])
    connection = connections["default"]
    _check_db(connection)
    _check_db(connection, suffix="1")
    try:
        _check_db(connection, suffix="2")
    except OperationalError:
        pass
    else:  # pragma: nocover
        pytest.fail("Database 2 should not exist here.")


def test_migrateci_pytest():
    execute_from_command_line(
        [
            "manage.py",
            "migrateci",
            "--parallel",
            "1",
            "--pytest",
        ]
    )
    connection = connections["default"]
    _check_db(connection)
    _check_db(connection, suffix="gw0")
    try:
        _check_db(connection, suffix="gw1")
    except OperationalError:
        pass
    else:  # pragma: nocover
        pytest.fail("Database gw1 should not exist here.")


def test_migrateci_cached(mocker):
    # Create empty cache file.
    basepath = Path(__file__).parent
    connection = connections["default"]
    shutil.copyfile(basepath / f"dump/{connection.vendor}.sql", "migrateci-default")
    setup_test_db_mock = mocker.patch("django_migrations_ci.django.setup_test_db")
    execute_from_command_line(["manage.py", "migrateci"])
    setup_test_db_mock.assert_not_called()
    _check_db(connections["default"])


def test_migrateci_local():
    execute_from_command_line(["manage.py", "migrateci", "--local"])
    _check_db(connections["default"])
    checksum = "b2bed1815363a843fdc8403d36497ddd"
    assert Path(f"migrateci-default-{checksum}").exists()



from subprocess import PIPE, Popen


def exec(command, env):
    if "test_test" in command:
        raise Exception(command)

    print("EXEC", command)
    p = Popen(
        command,
        shell=True,
        stdin=PIPE,
        stdout=PIPE,
        stderr=PIPE,
        env=env,
    )
    stdout, stderr = p.communicate()

    if stderr:
        print("EXEC ERROR", command, stdout, stderr)
    return stdout, stderr



from pathlib import Path

from django.core.management.base import BaseCommand

try:
    from django.test.runner import get_max_test_processes
except ImportError:
    # Django<4
    def get_max_test_processes():
        raise Exception(
            "Django<4 do not implement get_max_test_processes."
            " Use --parallel $(nproc) to not depend on this."
        )


from django_migrations_ci import django


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("-n", "--parallel", default=None)
        parser.add_argument(
            "--pytest",
            dest="is_pytest",
            action="store_true",
            default=False,
        )
        parser.add_argument("--directory", default="")
        parser.add_argument("--local", action="store_true", default=False)

    def handle(
        self, *args, parallel, is_pytest, local, directory, verbosity, **options
    ):
        if parallel == "auto":
            parallel = get_max_test_processes()
        elif parallel is not None:
            parallel = int(parallel)

        suffix = ""
        if local:
            suffix = f"-{django.hash_files()}"

        unique_connections = django.get_unique_connections()
        cached_files = {
            connection.alias: Path(directory) / f"migrateci-{connection.alias}{suffix}"
            for connection in unique_connections
        }

        if all(f.exists() for f in cached_files.values()):
            print("Database cache exists.")
            django.create_test_db(verbosity=verbosity)

            for connection in unique_connections:
                cached_file = cached_files[connection.alias]
                with django.test_db(connection):
                    django.load(connection, cached_file)
        else:
            print("Database cache does not exist.")
            django.setup_test_db(verbosity=verbosity)

            for connection in unique_connections:
                cached_file = cached_files[connection.alias]
                with django.test_db(connection):
                    django.dump(connection, cached_file)

        if parallel:
            for connection in unique_connections:
                with django.test_db(connection):
                    django.clone_test_db(
                        connection=connection,
                        parallel=parallel,
                        is_pytest=is_pytest,
                        verbosity=verbosity,
                    )



import pytest

from django_migrations_ci import django


@pytest.mark.parametrize(
    "db_name,expected_db_name",
    [
        ("db", "db"),
        ("db_gw0", "db_gw0"),
        ("db_gw0.sqlite3", "db.sqlite3_gw0"),
        ("db_gw0..sqlite3", "db.sqlite3_gw0"),
    ],
)
def test_fix_sqlite_pytest_suffix(db_name, expected_db_name):
    assert django._fix_sqlite_pytest_suffix(db_name) == expected_db_name


@pytest.mark.parametrize(
    "db_name,expected_db_name",
    [
        ("db", "db"),
        ("db_1", "db_1"),
        ("db_1.sqlite3", "db_1.sqlite3"),
        ("db_gw0.sqlite3", "db.sqlite3_gw0"),
        ("db_gw0..sqlite3", "db.sqlite3_gw0"),
        ("db_gw12.sqlite3", "db.sqlite3_gw12"),
        # Django<4 bugs.
        ("db_1..sqlite3", "db_1..sqlite3"),
        ("db_1.", "db_1."),
    ],
)
def test_transform_sqlite_name(db_name, expected_db_name):
    assert django._transform_sqlite_db_name(db_name) == expected_db_name



import os
from pathlib import Path
import tempfile

import pytest

from django_migrations_ci import django


@pytest.fixture(scope="session", autouse=True)
def setup_env():
    os.environ["DJANGO_SETTINGS_MODULE"] = "django_migrations_ci.tests.testapp.settings"


def _rm(pathname):
    for filename in Path(".").glob(pathname):
        Path(filename).unlink()


@pytest.fixture(autouse=True)
def remove_cached_files():
    pathname = "migrateci-*"
    _rm(pathname)
    yield
    _rm(pathname)


@pytest.fixture(autouse=True)
def remove_sqlite3_files():
    pathname = "dbtest*.sqlite3*"
    _rm(pathname)
    yield
    _rm(pathname)


@pytest.fixture(autouse=True)
def drop_postgresql_test_databases():
    for connection in django.get_unique_connections():
        if connection.vendor != "postgresql":
            continue
        with connection.cursor() as cursor:
            cursor.execute("select datname FROM pg_database")
            dbs = {db for db, in cursor.fetchall()}

        for db in dbs:
            if db.startswith("test_"):
                connection.creation._destroy_test_db(db, verbosity=True)


@pytest.fixture(autouse=True)
def drop_test_databases(remove_sqlite3_files, drop_postgresql_test_databases):
    pass


@pytest.fixture
def tempdir():
    directory = tempfile.TemporaryDirectory()
    with directory:
        yield directory.name



import os

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": os.getenv("POSTGRES_DB", "postgres"),
        "USER": os.getenv("POSTGRES_USER", "postgres"),
        "PASSWORD": os.getenv("POSTGRES_PASSWORD", "postgres"),
        "HOST": os.getenv("POSTGRES_HOST", "localhost"),
        "PORT": os.getenv("POSTGRES_PORT", "5432"),
    },
}
